unit pdfhelpers;

{$mode ObjFPC}{$H+}

interface

uses
  Classes, SysUtils,  fpTTF, fpPDF;

type

  TPDFAlign = (alLeft, alRight, alCenter);

  { TPDFPageHelper }

  TPDFPageHelper = class helper for TPDFPage
  public
    function GetStdFontCharWidthsArray(const AFontName: string):
      TPDFFontWidthArray;
    function GetStdFontDescender(Font: TPDFEmbeddedFont): single;
    function GetStdFontTextHeight(Font: TPDFEmbeddedFont): single;
    function GetStdFontTextWidth(AString: string; Font: TPDFEmbeddedFont): single;
    function GetTextWidthMmNoCheck(AText: utf8string): TPDFFloat;
    function GetPaperWidth: TPDFFloat;
    procedure GetTextDimsMm(AText: utf8string; out W, H, Desc: TPDFFloat);
    procedure SetFontSize(Size: integer);
  end;

  { TPDFDocumentHelper }

  TPDFDocumentHelper = class helper for TPDFDocument
  public
    function AddSystemFont(const AFontName: string; ABold: boolean;
      AItalic: boolean): integer;
  end;

implementation

//uses LConvEncoding;

{$I fontmetrics_stdpdf.inc }

resourcestring
  rsErrNoFontDefined = 'No Font was set - please use SetFont() first.';
  rsErrUnknownStdFont = 'Unknown standard PDF font name <%s>.';

{ TPDFDocumentHelper }

function TPDFDocumentHelper.AddSystemFont(const AFontName: string;
  ABold: boolean; AItalic: boolean): integer;
var
  AFontFile, APSName: string;
  lNewFC, aBaseFont: TFPFontCacheItem;
begin
  aBaseFont := gTTFontCache.FindFont(aFontName);
  if not Assigned(aBaseFont) then
    raise Exception.Create('Basefont ' + AFontName + ' not found!');
  // find corresponding font style (bold and/or italic)
  lNewFC := gTTFontCache.Find(aBaseFont.FamilyName, aBold, aItalic);
  if not Assigned(lNewFC) then
    raise Exception.Create('Font Style not found!');
  APSName := lNewFC.PostScriptName;
  AFontFile := gTTFontCache.Find(APSName).FileName;
  Result := AddFont(AFontFile, APSName);
end;

{ TPDFPageHelper }

procedure TPDFPageHelper.GetTextDimsMm(AText: utf8string; out W, H, Desc: TPDFFloat);
var
  lFC: TFPFontCacheItem;
  lHeight, lDescenderHeight, lWidth: single;
  FontName: string;
begin
  if not Assigned(LastFont) then
    raise EPDF.Create(rsErrNoFontDefined);
  FontName := Document.Fonts[LastFont.FontIndex].Name;
  if Document.IsStandardPDFFont(FontName) then
  begin
    W := (GetStdFontTextWidth(UTF8Decode(AText), LastFont) * 25.4) / gTTFontCache.DPI;
    H := (GetStdFontTextHeight(LastFont) * 25.4) / gTTFontCache.DPI;
    Desc := (GetStdFontDescender(LastFont) * 25.4) / gTTFontCache.DPI;
  end
  else
  begin
    lFC := gTTFontCache.Find(FontName);
    if not Assigned(lFC) then
      raise Exception.Create(FontName + ' font not found');
    lHeight := lFC.TextHeight(AText, LastFont.PointSize, lDescenderHeight);
    H := (lHeight * 25.4) / gTTFontCache.DPI;
    Desc := (lDescenderHeight * 25.4) / gTTFontCache.DPI;
    lWidth := lFC.TextWidth(AText, LastFont.PointSize);
    W := (lWidth * 25.4) / gTTFontCache.DPI;
  end;
end;

function TPDFPageHelper.GetTextWidthMmNoCheck(AText: utf8string): TPDFFloat;
var
  lFC: TFPFontCacheItem;
  FontName: string;
begin
  FontName := Document.Fonts[LastFont.FontIndex].Name;
  if Document.IsStandardPDFFont(FontName) then
    Result := (GetStdFontTextWidth(UTF8Decode(AText), LastFont) * 25.4) /
      gTTFontCache.DPI
  else
  begin
    lFC := gTTFontCache.Find(FontName);
    Result := (lFC.TextWidth(AText, LastFont.PointSize) * 25.4) / gTTFontCache.DPI;
  end;
end;

function TPDFPageHelper.GetStdFontCharWidthsArray(const AFontName: string):
TPDFFontWidthArray;
begin
  case AFontName of
    'Courier': Result := FONT_COURIER_FULL;
    'Courier-Bold': Result := FONT_COURIER_FULL;
    'Courier-Oblique': Result := FONT_COURIER_FULL;
    'Courier-BoldOblique': Result := FONT_COURIER_FULL;
    'Helvetica': Result := FONT_HELVETICA_ARIAL;
    'Helvetica-Bold': Result := FONT_HELVETICA_ARIAL_BOLD;
    'Helvetica-Oblique': Result := FONT_HELVETICA_ARIAL_ITALIC;
    'Helvetica-BoldOblique': Result := FONT_HELVETICA_ARIAL_BOLD_ITALIC;
    'Times-Roman': Result := FONT_TIMES;
    'Times-Bold': Result := FONT_TIMES_BOLD;
    'Times-Italic': Result := FONT_TIMES_ITALIC;
    'Times-BoldItalic': Result := FONT_TIMES_BOLD_ITALIC;
    'Symbol': Result := FONT_SYMBOL;
    'ZapfDingbats': Result := FONT_ZAPFDINGBATS;
    else
      raise EPDF.CreateFmt(rsErrUnknownStdFont, [AFontName]);
  end;
end;

function TPDFPageHelper.GetStdFontTextWidth(AString: string;
  Font: TPDFEmbeddedFont): single;
var
  i: integer;
  lWidth: double;
  lFontName: string;
begin
  lFontName := Document.Fonts[Font.FontIndex].Name;
  lWidth := 0;
  for i := 1 to Length(AString) do
    lWidth := lWidth + GetStdFontCharWidthsArray(lFontName)[Ord(AString[i])];
  Result := lWidth * Font.PointSize / 1540;
end;

function TPDFPageHelper.GetStdFontTextHeight(Font: TPDFEmbeddedFont): single;
var
  lFontName: string;
begin
  lFontName := Document.Fonts[Font.FontIndex].Name;
  Result := 0;
  case lFontName of
    'Courier': Result := FONT_TIMES_COURIER_CAPHEIGHT;
    'Courier-Bold': Result := FONT_TIMES_COURIER_CAPHEIGHT;
    'Courier-Oblique': Result := FONT_TIMES_COURIER_CAPHEIGHT;
    'Courier-BoldOblique': Result := FONT_TIMES_COURIER_CAPHEIGHT;
    'Helvetica': Result := FONT_HELVETICA_ARIAL_CAPHEIGHT;
    'Helvetica-Bold': Result := FONT_HELVETICA_ARIAL_BOLD_CAPHEIGHT;
    'Helvetica-Oblique': Result := FONT_HELVETICA_ARIAL_ITALIC_CAPHEIGHT;
    'Helvetica-BoldOblique': Result := FONT_HELVETICA_ARIAL_BOLD_ITALIC_CAPHEIGHT;
    'Times-Roman': Result := FONT_TIMES_CAPHEIGHT;
    'Times-Bold': Result := FONT_TIMES_BOLD_CAPHEIGHT;
    'Times-Italic': Result := FONT_TIMES_ITALIC_CAPHEIGHT;
    'Times-BoldItalic': Result := FONT_TIMES_BOLD_ITALIC_CAPHEIGHT;
    'Symbol': Result := 300;
    'ZapfDingbats': Result := 300;
    else
      raise EPDF.CreateFmt(rsErrUnknownStdFont, [lFontName]);
  end;
  Result := Result * Font.PointSize / 1540;
end;

function TPDFPageHelper.GetStdFontDescender(Font: TPDFEmbeddedFont): single;
var
  lFontName: string;
begin
  lFontName := Document.Fonts[Font.FontIndex].Name;
  Result := 0;
  case lFontName of
    'Courier': Result := FONT_TIMES_COURIER_DESCENDER;
    'Courier-Bold': Result := FONT_TIMES_COURIER_DESCENDER;
    'Courier-Oblique': Result := FONT_TIMES_COURIER_DESCENDER;
    'Courier-BoldOblique': Result := FONT_TIMES_COURIER_DESCENDER;
    'Helvetica': Result := FONT_HELVETICA_ARIAL_DESCENDER;
    'Helvetica-Bold': Result := FONT_HELVETICA_ARIAL_BOLD_DESCENDER;
    'Helvetica-Oblique': Result := FONT_HELVETICA_ARIAL_ITALIC_DESCENDER;
    'Helvetica-BoldOblique': Result := FONT_HELVETICA_ARIAL_BOLD_ITALIC_DESCENDER;
    'Times-Roman': Result := FONT_TIMES_DESCENDER;
    'Times-Bold': Result := FONT_TIMES_BOLD_DESCENDER;
    'Times-Italic': Result := FONT_TIMES_ITALIC_DESCENDER;
    'Times-BoldItalic': Result := FONT_TIMES_BOLD_ITALIC_DESCENDER;
    'Symbol': Result := 300;
    'ZapfDingbats': Result := 300;
    else
      raise EPDF.CreateFmt(rsErrUnknownStdFont, [lFontName]);
  end;
  Result := Result * Font.PointSize / 1540;
end;

procedure TPDFPageHelper.SetFontSize(Size: integer);
begin
  if Assigned(LastFont) then SetFont(LastFont.FontIndex, Size);
end;

function TPDFPageHelper.GetPaperWidth: TPDFFloat;
begin
  case UnitOfMeasure of
    uomMillimeters:
    begin
      Result := PDFtoMM(Paper.W);
    end;
    uomCentimeters:
    begin
      Result := PDFtoCM(Paper.W);
    end;
    uomInches:
    begin
      Result := PDFtoInches(Paper.W);
    end;
    uomPixels:
    begin
      Result := Paper.W;
    end;
  end;
end;

end.
